from slip.dbus import polkit
import dbus
import gobject

import gtk
import gettext
DOMAIN = "system-config-kdump"
gtk.glade.bindtextdomain(DOMAIN)
_ = lambda x: gettext.ldgettext(DOMAIN, x)
N_ = lambda x: x

import dbus.mainloop.glib

# all needed for Python-slip, PoilcyKit and dbus
class DBusProxy (gobject.GObject):
    """
    Class used for communication with/via dbus
    """

    def __init__ (self, progress_window):
        gobject.GObject.__init__(self)
        dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
        self.bus = dbus.SystemBus ()
        self.dbus_object = self.bus.get_object (
            "org.fedoraproject.systemconfig.kdump.mechanism",
            "/org/fedoraproject/systemconfig/kdump/object")

        self.progress_window = progress_window

        # What was on stdout
        self.std = ""

        # What was on stderr
        self.err = ""

        # Which command failed
        self.cmd = ""

        # return code
        self.retcode = 0

        # a list returned from some dbus call
        self.a_list = []

        # a string returned from some dbus call
        self.a_string = ""

        # a dictionary returned from some dbus call
        self.a_dict = {}

        self.a_bool = False
        self.a_int = 0
        self.kdump_mem = 0
        self.kdump_offset = 0
        self.target = (None, None)

        gobject.signal_new("proxy-error", self, gobject.SIGNAL_RUN_FIRST,
            gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,))

        self.loop = gobject.MainLoop()

    def handle_reply(self, (cmd, retcode, std, err)):
        self.retcode = retcode
        self.std = std
        self.err = err
        self.cmd = cmd
        self.loop.quit()
        self.progress_window.hide()

    def handle_list_reply(self, a_list):
        self.a_list = a_list
        self.loop.quit()
        self.progress_window.hide()

    def handle_string_reply(self, a_string):
        self.a_string = a_string
        self.loop.quit()
        self.progress_window.hide()

    def handle_dict_reply(self, a_dict):
        self.a_dict = a_dict
        self.loop.quit()
        self.progress_window.hide()

    def handle_totalmem_reply(self, (acc, tm)):
        self.a_bool = acc
        self.a_int = tm
        self.loop.quit()
        self.progress_window.hide()

    def handle_kdumpmem_reply(self, (mem, offset)):
        self.kdump_mem = mem
        self.kdump_offset = offset
        self.loop.quit()
        self.progress_window.hide()

    def handle_target_reply(self, (target_type, target_path)):
        self.target = (target_type, target_path)
        self.loop.quit()
        self.progress_window.hide()

    def handle_reply_config(self, (retcode, err)):
        self.retcode = retcode
        self.err = err
        self.loop.quit()
        self.progress_window.hide()

    def handle_bool_reply(self, status):
        self.a_bool = status
        self.loop.quit()
        self.progress_window.hide()

    def handle_error(self, exception):
        self.loop.quit()
        self.progress_window.hide()
        self.emit("proxy-error", exception)
        self.retcode = -1
        self.std = ""
        self.err = ""
        self.cmd = ""

    @polkit.enable_proxy
    def getkdumpmem (self):
        """ Return how much memory is reserved for kdump  """
        self.progress_window.set_label(_("Getting kdump reserved memory"))
        self.progress_window.show()
        self.dbus_object.getkdumpmem(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_kdumpmem_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return (self.kdump_mem, self.kdump_offset)



    @polkit.enable_proxy
    def gettotalmem (self):
        """ Get total memory on system  """
        self.progress_window.set_label(_("Getting system total memory"))
        self.progress_window.show()
        self.dbus_object.gettotalmem(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_totalmem_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return (self.a_bool, self.a_int)

    @polkit.enable_proxy
    def getarch (self):
        """ Return architecture the backend is running on  """
        self.progress_window.set_label(_("Getting architecture"))
        self.progress_window.show()
        self.dbus_object.getarch(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_string_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_string

    @polkit.enable_proxy
    def getrunningkernel (self):
        """ Return kernel the backend is running on"""
        self.progress_window.set_label(_("Getting running kernel"))
        self.progress_window.show()
        self.dbus_object.getrunningkernel(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_string_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_string

    @polkit.enable_proxy
    def getdefaultkernel (self):
        """
        Return name of default kernel set in bootloader configuration string
        """
        self.progress_window.set_label(_("Getting default kernel name"))
        self.progress_window.show()
        self.dbus_object.getdefaultkernel(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_string_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_string

    @polkit.enable_proxy
    def getcmdline (self, kernel):
        """
        Return command line arguments for specific kernel
        """
        self.progress_window.set_label(_("Getting argument for kernel"))
        self.progress_window.show()
        self.dbus_object.getcmdline(
            kernel,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_dict_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_dict

    @polkit.enable_proxy
    def getxencmdline (self, kernel):
        """
        Return command line arguments for specific xen kernel
        """
        self.progress_window.set_label(_("Getting argument for xen kernel"))
        self.progress_window.show()
        self.dbus_object.getxencmdline(
            kernel,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.cmd, self.retcode, self.std, self.err

    @polkit.enable_proxy
    def getallkernels (self):
        """
        Return a list of all kernels in bootloader configuration files
        """
        self.progress_window.set_label(_("Getting name of all kernels"))
        self.progress_window.show()
        self.dbus_object.getallkernels(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_list_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_list

    @polkit.enable_proxy
    def getdefaultaction (self):
        """
        Return a default action
        """
        self.progress_window.set_label(_("Getting default action"))
        self.progress_window.show()
        self.dbus_object.getdefaultaction(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_string_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_string

    @polkit.enable_proxy
    def getpath (self):
        """
        Return a path
        """
        self.progress_window.set_label(_("Getting path"))
        self.progress_window.show()
        self.dbus_object.getpath(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_string_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_string

    @polkit.enable_proxy
    def getcollector (self):
        """
        Return a default action
        """
        self.progress_window.set_label(_("Getting collector settings"))
        self.progress_window.show()
        self.dbus_object.getcollector(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_string_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_string

    @polkit.enable_proxy
    def gettarget (self):
        """
        Return a target settings
        """
        self.progress_window.set_label(_("Getting target settings"))
        self.progress_window.show()
        self.dbus_object.gettarget(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_target_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.target



    @polkit.enable_proxy
    def writedumpconfig (self, config_string):
        """
        Write config_string string to kdump.conf file and returns error code
        and reread string
        """
        self.progress_window.set_label(_("Writing kdump configuration"))
        self.progress_window.show()
        self.dbus_object.writedumpconfig(
            config_string,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply_config,
            error_handler = self.handle_error)
        self.loop.run()
        return self.retcode, self.err


    @polkit.enable_proxy
    def writebootconfig (self, config_string):
        """
        Update bootloader configuration. config_string is grubby arguments.
        """
        self.progress_window.set_label(_("Writing bootloader configuration"))
        self.progress_window.show()
        self.dbus_object.writebootconfig(
            config_string,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.cmd, self.retcode, self.std, self.err


    @polkit.enable_proxy
    def handlekdumpservice(self, chkconfig_status, service_op):
        """
        Start or stop kdump service, turn on or off init script
        """
        self.progress_window.set_label(_("Handling kdump service"))
        self.progress_window.show()
        self.dbus_object.handledumpservice (
            (chkconfig_status, service_op),
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.cmd, self.retcode, self.std, self.err

    @polkit.enable_proxy
    def getfstab(self):
        """
        Get entries from fstab, local disks
        """
        self.progress_window.set_label(_("Getting local file systems"))
        self.progress_window.show()
        self.dbus_object.getfstab(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_dict_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_dict

    @polkit.enable_proxy
    def setpath(self, path):
        self.progress_window.set_label(_("Setting path"))
        self.progress_window.show()
        self.dbus_object.setpath(
            path,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_bool_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_bool

    @polkit.enable_proxy
    def setkernelargs(self, kernel, kargs):
        self.progress_window.set_label(_("Setting kernel arguments"))
        self.progress_window.show()
        self.dbus_object.setkernelargs(
            kernel, kargs,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_bool_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_bool


    @polkit.enable_proxy
    def save(self):
        self.progress_window.set_label(_("Saving kdump configuration"))
        self.progress_window.show()
        self.dbus_object.save(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_bool_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_bool

    @polkit.enable_proxy
    def setdefaultaction(self, action):
        self.progress_window.set_label(_("Setting default action"))
        self.progress_window.show()
        self.dbus_object.setdefaultaction(
            action,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_bool_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_bool

    @polkit.enable_proxy
    def setcollector(self, collector):
        self.progress_window.set_label(_("Setting core collector"))
        self.progress_window.show()
        self.dbus_object.setcollector(
            collector,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_bool_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_bool

    @polkit.enable_proxy
    def unsettarget(self):
        self.progress_window.set_label(_("Unsetting target"))
        self.progress_window.show()
        self.dbus_object.unsettarget(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_bool_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_bool


    @polkit.enable_proxy
    def unsetcrashkernel(self, kernel):
        self.progress_window.set_label(_("Unsetting crashkernel"))
        self.progress_window.show()
        self.dbus_object.unsetcrashkernel(
            kernel,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_bool_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_bool


    @polkit.enable_proxy
    def unsetdefault(self):
        self.progress_window.set_label(_("Unsetting default action"))
        self.progress_window.show()
        self.dbus_object.unsetdefault(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_bool_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_bool



    @polkit.enable_proxy
    def setlocal(self, fs, partition):
        self.progress_window.set_label(_("Setting local target"))
        self.progress_window.show()
        self.dbus_object.setlocal(
            (fs, partition),
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_bool_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_bool

    @polkit.enable_proxy
    def setssh(self, user, server):
        self.progress_window.set_label(_("Setting ssh target"))
        self.progress_window.show()
        self.dbus_object.setssh(
            (user, server),
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_bool_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_bool

    @polkit.enable_proxy
    def setnfs(self, nfs):
        self.progress_window.set_label(_("Setting nfs target"))
        self.progress_window.show()
        self.dbus_object.setnfs(
            nfs,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_bool_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_bool

    @polkit.enable_proxy
    def setrawdevice(self, devname):
        self.progress_window.set_label(_("Setting raw device target"))
        self.progress_window.show()
        self.dbus_object.setrawdevice(
            devname,
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_bool_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_bool



    @polkit.enable_proxy
    def getrawdevices (self):
        """
        Return a list of raw devices got from /proc/partitions
        """
        self.progress_window.set_label(_("Getting raw devices"))
        self.progress_window.show()
        self.dbus_object.getrawdevices(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_list_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.a_list


    @polkit.enable_proxy
    def getservicestatus(self):
        """
        Get the current status of the kdump service
        """
        self.progress_window.set_label(_("Getting status of kdump service"))
        self.progress_window.show()
        self.dbus_object.getservicestatus(
            dbus_interface = "org.fedoraproject.systemconfig.kdump.mechanism",
            reply_handler = self.handle_reply,
            error_handler = self.handle_error)
        self.loop.run()
        return self.cmd, self.retcode, self.std, self.err

